

#include "Keyboard.H"
#include "Led.H"


using namespace std;
using namespace avelino;


#define  RCC_APB2ENR           *((volatile uint32_t *) 0x40021018)
#define  GPIOB_CRL             *((volatile uint32_t *) 0x40010C00)
#define  GPIOB_CRH             *((volatile uint32_t *) 0x40010C04)
#define  GPIOB_ODR             *((volatile uint32_t *) 0x40010C0C)
#define  EXTI_IMR              *((volatile uint32_t *) 0x40010400)
#define  EXTI_EMR              *((volatile uint32_t *) 0x40010404)
#define  EXTI_RTSR             *((volatile uint32_t *) 0x40010408)
#define  EXTI_FTSR             *((volatile uint32_t *) 0x4001040C)
#define  EXTI_PR               *((volatile uint32_t *) 0x40010414)
#define  AFIO_EXTICR2          *((volatile uint32_t *) 0x4001000C)
#define  AFIO_EXTICR4          *((volatile uint32_t *) 0x40010014)
#define  NVIC_ISER             ((volatile uint32_t *) 0xE000E100)
#define  NVIC_ENABLE_IRQ(irq)  NVIC_ISER[(irq) >> 5] |= ((uint32_t) 1) << ((irq) & 0x0000001F)


KeyboardListener *Keyboard::listener = nullptr;
Keyboard::Status Keyboard::status = Keyboard::Status::IDLE;
int8_t Keyboard::currentKey = 0;
int32_t Keyboard::tickCounter;


void Keyboard::init(KeyboardListener *listener) {
	Keyboard::status = Status::IDLE;
	// clock enable para puerto B
	RCC_APB2ENR |= ((uint32_t) 1) << 3;
	// configurar GPIOS B4 y B5 como entrada libre (alta impedancia)
	GPIOB_CRL = (GPIOB_CRL & 0xFF00FFFF) | 0x00440000;
	EXTI_IMR |= 0x00000030;   // bits 4 y 5
	EXTI_RTSR &= 0xFFFFFFCF;  // deshabilitar interrupción por flanco de subida
	EXTI_FTSR |= 0x00000030;  // habilitar interrupción por flanco de bajada
	RCC_APB2ENR |= 0x00000001;                                   // habilitar AF para poder usar AFIO_EXTICR2
	AFIO_EXTICR2 = (AFIO_EXTICR2 & 0xFFFFFF00) | 0x00000011;     // EXTI4 y EXTI5 asociadas al puerto B
	Keyboard::listener = listener;
	NVIC_ENABLE_IRQ(10);      // habilitar interrupciones
	NVIC_ENABLE_IRQ(23);
}


void Keyboard::interrupt() {
	if (EXTI_PR & 0x00000010) {
		Keyboard::currentKey = 1;
		EXTI_PR = 0x00000010;
	}
	else if (EXTI_PR & 0x00000020) {
		Keyboard::currentKey = 2;
		EXTI_PR = 0x00000020;
	}
}


void Keyboard::tick() {
	Status localStatus = Keyboard::status;
	do {
		Keyboard::status = localStatus;
		if (Keyboard::status == Status::IDLE) {
			if (Keyboard::currentKey != 0) {
				if (Keyboard::listener != nullptr)
					Keyboard::listener->buttonPressed(Keyboard::currentKey);
				Keyboard::tickCounter = 200;
				localStatus = Status::WAIT_AFTER_PRESSED;
			}
		}
		else if (Keyboard::status == Status::WAIT_AFTER_PRESSED) {
			Keyboard::tickCounter--;
			if (Keyboard::tickCounter <= 0) {
				Keyboard::currentKey = 0;
				localStatus = Status::IDLE;
			}
		}
	} while (localStatus != Keyboard::status);
}

