

#ifndef  __SN76489_H__
#define  __SN76489_H__


#include <stdint.h>
#include "StaticQueue.H"


extern "C++" {
	namespace avelino {
		using namespace std;

		class SN76489Interface {
			public:
				// assumes CE = 0 always
				virtual void sn76489SetByte(uint8_t v) = 0;
				virtual void sn76489SetWE(uint8_t v) = 0;
				virtual bool sn76489GetReady() = 0;
		};

		class SN76489 {
			protected:
				SN76489Interface *interface;
				class CommandWithDelay {
					public:
						uint8_t command;
						int32_t delayTicks;
						CommandWithDelay(uint8_t command, int32_t delayTicks) : command(command), delayTicks(delayTicks) { };
						CommandWithDelay(uint8_t command) : command(command), delayTicks(0) { };
						CommandWithDelay() : command(0), delayTicks(0) { };
				};
				StaticQueue<CommandWithDelay, 64> txQueue;
				enum class Status : int8_t { WAIT_FOR_INIT, IDLE, WAIT_FOR_READY, WAIT_FOR_NOT_READY, WAIT_FOR_DELAY, WAIT_FOR_READY_2, WAIT_FOR_DELAY_2, WAIT_DELAY_TICKS };
				Status status;
				int32_t tickCounter;
				int32_t delayTicks;
			public:
				SN76489() : interface (nullptr), status(Status::IDLE) { };
				enum class NoiseGeneratorFrequency : uint8_t { N_512 = 0, N_1024 = 1, N_2048 = 2, TONE_GENERATOR_3 = 3 };
				enum class NoiseGeneratorFeedback : uint8_t { PERIODIC = 0, WHITE = 1 };
				void init(SN76489Interface *interface);
				void setToneGeneratorFrequency(uint8_t index, uint16_t frequency, int16_t sweepFrequencyIncrement = 0, int16_t sweepTicksPerIncrement = 0, int16_t sweepNumIncrements = 0);
				void setToneGeneratorAttenuation(uint8_t index, uint8_t attenuation);
				void setNoiseGeneratorFrequency(NoiseGeneratorFrequency frequency, NoiseGeneratorFeedback feedback);
				void setNoiseGeneratorAttenuation(uint8_t attenuation, int32_t delayTicks = 0);
				void tick();
		};
	}
}


#endif  // __SN76489_H__
