

#include "SN76489.H"
#include "Led.H"


using namespace avelino;
using namespace std;


void SN76489::init(SN76489Interface *interface) {
	this->interface = interface;
	this->tickCounter = 100;
	this->status = Status::WAIT_FOR_INIT;
	this->interface->sn76489SetWE(1);
}


void SN76489::setToneGeneratorFrequency(uint8_t index, uint16_t frequency, int16_t sweepFrequencyIncrement, int16_t sweepTicksPerIncrement, int16_t sweepNumIncrements) {
	if (index > 2)        // 0, 1 or 2
		index = 2;
	frequency &= 0x03FF;  // 10 bit value
	//this->txQueue.push(0x80 | (index << 5) | ((frequency >> 6) & 0x0F));
	//this->txQueue.push(frequency & 0x3F);
	this->txQueue.push({(uint8_t) (0x80 | (index << 5) | (frequency & 0x0F))});
	this->txQueue.push({(uint8_t) ((frequency >> 4) & 0x3F), sweepTicksPerIncrement});
	while (sweepNumIncrements > 0) {
		frequency += sweepFrequencyIncrement;
		sweepNumIncrements--;
		this->txQueue.push({(uint8_t) ((frequency >> 4) & 0x3F), sweepTicksPerIncrement});
	}
}


void SN76489::setToneGeneratorAttenuation(uint8_t index, uint8_t attenuation) {
	if (index > 2)        // 0, 1 or 2
		index = 2;
	this->txQueue.push({(uint8_t) (0x80 | (index << 5) | 0x10 | (attenuation & 0x0F))});
}


void SN76489::setNoiseGeneratorFrequency(NoiseGeneratorFrequency frequency, NoiseGeneratorFeedback feedback) {
	this->txQueue.push({(uint8_t) (0xE0 | ((static_cast<uint8_t>(feedback) & 0x01) << 2) | (static_cast<uint8_t>(feedback) & 0x03))});
}


void SN76489::setNoiseGeneratorAttenuation(uint8_t attenuation, int32_t delayTicks) {
	this->txQueue.push({(uint8_t) (0xF0 | (attenuation & 0x0F)), delayTicks});
}


void SN76489::tick() {
	Status localStatus = this->status;
	do {
		this->status = localStatus;
		if (this->status == Status::WAIT_FOR_INIT) {
			this->tickCounter--;
			if (this->tickCounter <= 0)
				localStatus = Status::IDLE;
		}
		if (this->status == Status::IDLE) {
			if (!this->txQueue.empty()) {
				localStatus = Status::WAIT_FOR_READY;
			}
		}
		else if (this->status == Status::WAIT_FOR_READY) {
			if (this->interface->sn76489GetReady()) {
				this->interface->sn76489SetByte(this->txQueue.head().command);
				this->delayTicks = this->txQueue.head().delayTicks;
				this->txQueue.pop();
				this->interface->sn76489SetWE(0);
				localStatus = Status::WAIT_FOR_NOT_READY;
			}
		}
		else if (this->status == Status::WAIT_FOR_NOT_READY) {
			if (!this->interface->sn76489GetReady()) {
				this->tickCounter = 1;
				localStatus = Status::WAIT_FOR_DELAY;
			}
		}
		else if (this->status == Status::WAIT_FOR_DELAY) {
			this->tickCounter--;
			if (this->tickCounter <= 0)
				localStatus = Status::WAIT_FOR_READY_2;
		}
		else if (this->status == Status::WAIT_FOR_READY_2) {
			if (this->interface->sn76489GetReady()) {
				this->interface->sn76489SetWE(1);
				this->tickCounter = 1;
				localStatus = Status::WAIT_FOR_DELAY_2;
			}
		}
		else if (this->status == Status::WAIT_FOR_DELAY_2) {
			this->tickCounter--;
			if (this->tickCounter <= 0)
				localStatus = Status::WAIT_DELAY_TICKS;
		}
		else if (this->status == Status::WAIT_DELAY_TICKS) {
			if (this->delayTicks <= 0)
				localStatus = Status::IDLE;
			else
				this->delayTicks--;
		}
	} while (localStatus != this->status);
}

