

#include <stdint.h>
#include "Led.H"
#include "SN76489.H"
#include "Keyboard.H"


using namespace std;
using namespace avelino;


#define  NVIC_ISER    ((volatile uint32_t *) 0xE000E100)
#define  NVIC_ICER    ((volatile uint32_t *) 0xE000E180)
#define  NVIC_IPR     ((volatile uint32_t *) 0xE000E400)
#define  RCC_APB2ENR  *((volatile uint32_t *) 0x40021018)
#define  RCC_APB1ENR  *((volatile uint32_t *) 0x4002101C)
#define  GPIOC_CRH    *((volatile uint32_t *) 0x40011004)
#define  GPIOC_ODR    *((volatile uint32_t *) 0x4001100C)
#define  SYST_CSR     *((volatile uint32_t *) 0xE000E010)
#define  SYST_RVR     *((volatile uint32_t *) 0xE000E014)
#define  SYST_CVR     *((volatile uint32_t *) 0xE000E018)
#define  TIM2_CR1     *((volatile uint16_t *) 0x40000000)
#define  TIM2_PSC     *((volatile uint16_t *) 0x40000028)
#define  TIM2_ARR     *((volatile uint16_t *) 0x4000002C)
#define  TIM2_CCER    *((volatile uint16_t *) 0x40000020)
#define  TIM2_CCMR1   *((volatile uint16_t *) 0x40000018)
#define  TIM2_CCR2    *((volatile uint16_t *) 0x40000038)
#define  USART_CR1    *((volatile uint32_t *) 0x4001380C)
#define  USART_BRR    *((volatile uint32_t *) 0x40013808)
#define  USART_DR     *((volatile uint32_t *) 0x40013804)
#define  USART_SR     *((volatile uint32_t *) 0x40013800)
#define  GPIOA_CRL    *((volatile uint32_t *) 0x40010800)
#define  GPIOA_CRH    *((volatile uint32_t *) 0x40010804)
#define  GPIOA_ODR    *((volatile uint32_t *) 0x4001080C)
#define  GPIOB_CRL    *((volatile uint32_t *) 0x40010C00)
#define  GPIOB_CRH    *((volatile uint32_t *) 0x40010C04)
#define  GPIOB_IDR    *((volatile uint32_t *) 0x40010C08)
#define  GPIOB_ODR    *((volatile uint32_t *) 0x40010C0C)


#define  NVIC_ENABLE_IRQ(irq)         NVIC_ISER[(irq) >> 5] |= ((uint32_t) 1) << ((irq) & 0x0000001F)
#define  NVIC_DISABLE_IRQ(irq)        NVIC_ICER[(irq) >> 5] |= ((uint32_t) 1) << ((irq) & 0x0000001F)
#define  NVIC_SET_PRIORITY(irq, pri)  NVIC_IPR[(irq) >> 2] |= (NVIC_IPR[(irq) >> 2] & ~(0x000000FF << (((irq) & 3) << 3))) | (((uint32_t) (pri)) << (((irq) & 3) << 3))


class SN76489GPIOInterface : public SN76489Interface {
	public:
		void init();
		virtual void sn76489SetByte(uint8_t v);
		virtual void sn76489SetWE(uint8_t v);
		virtual bool sn76489GetReady();
};


void SN76489GPIOInterface::init() {
	// enable clock on port A and B
	RCC_APB2ENR |= ((uint32_t) 1) << 2;
	RCC_APB2ENR |= ((uint32_t) 1) << 3;
	// data bus on PA (push-pull output max. 2 MHz)
	GPIOA_CRL = 0x22222222;
	// /WE signal on PB1 (push-pull output max. 2 MHz)
	GPIOB_CRL = (GPIOB_CRL & 0xFFFFFF0F) | 0x00000020;
	// READY signal on PB10 (pull up-down input, 5v tolerant)
	GPIOB_CRH = (GPIOB_CRH & 0xFFFFF0FF) | 0x00000800;
	GPIOB_ODR |= (((uint32_t) 1) << 10);   // pull-up, because READY signal is open collector
}


void SN76489GPIOInterface::sn76489SetByte(uint8_t v) {
	GPIOA_ODR = (GPIOA_ODR & 0xFF00) | ((uint32_t) v);
}


void SN76489GPIOInterface::sn76489SetWE(uint8_t v) {
	GPIOB_ODR = (GPIOB_ODR & 0xFFFD) | (((uint32_t) v) << 1);
}


bool SN76489GPIOInterface::sn76489GetReady() {
	return ((GPIOB_IDR & 0x0400) != 0);
}


SN76489 sn76489;
SN76489GPIOInterface sn76489GPIOInterface;


void systick() __attribute__ ((section(".systick")));


volatile uint32_t tickCounter = 0;


void systick() {
	sn76489.tick();
	Keyboard::tick();
	/*
	tickCounter++;
	if (tickCounter == 200) {
		ledToggle();
		tickCounter = 0;
	}
	*/
}


class ButtonListener : public KeyboardListener {
	public:
		virtual void buttonPressed(int8_t key);
};


void ButtonListener::buttonPressed(int8_t key) {
	//Led::toggle();
	if (key == 1) {
		sn76489.setToneGeneratorAttenuation(0, 0);   // full volume
		sn76489.setToneGeneratorFrequency(0, 477, 100, 5, 10);
		sn76489.setToneGeneratorAttenuation(0, 0x0F);   // min volume
	}
	else if (key == 2) {
		sn76489.setNoiseGeneratorFrequency(SN76489::NoiseGeneratorFrequency::N_512, SN76489::NoiseGeneratorFeedback::WHITE);
		for (int i = 0; i < 17; i++) {
			sn76489.setNoiseGeneratorAttenuation(0, 20);
			sn76489.setNoiseGeneratorAttenuation(0xFF, 30);
		}
	}
}


ButtonListener buttonListener;


int main() {
	Led::init();
	Led::set(false);
	Keyboard::init(&buttonListener);
	// enable systick timer
	SYST_RVR = 0x0001FFFF;
	SYST_CVR = 0;
	SYST_CSR = 7;
	// sn76489
	sn76489GPIOInterface.init();
	sn76489.init(&sn76489GPIOInterface);    // c minor = C, D#, G = 262, 311, 392
	/*  C min chord
	sn76489.setToneGeneratorAttenuation(0, 0);   // full volume
	sn76489.setToneGeneratorFrequency(0, 477);   // 4000000 / (32 * 262) = 477
	sn76489.setToneGeneratorAttenuation(1, 0);   // full volume
	sn76489.setToneGeneratorFrequency(1, 402);   // 4000000 / (32 * 311) = 402
	sn76489.setToneGeneratorAttenuation(2, 0);   // full volume
	sn76489.setToneGeneratorFrequency(2, 319);   // 4000000 / (32 * 392) = 319
	*/
	sn76489.setToneGeneratorAttenuation(0, 0xFF); // min volume
	sn76489.setToneGeneratorAttenuation(1, 0x0F); // min volume
	sn76489.setToneGeneratorAttenuation(2, 0x0F); // min volume
	sn76489.setNoiseGeneratorAttenuation(0x0F);   // min volume for noise generator
	//sn76489.setToneGeneratorFrequency(0, 477, 100, 5, 10);
	//sn76489.setToneGeneratorAttenuation(0, 0x0F);   // min volume
	while (true) {
		// WFI (wait for interrupt) instruction, enters low power mode
		asm volatile ("wfi");
	}
}
